part of lunaris;

class Station {
  final player = new Player();
  final drones = new List<Drone>();
  bool created = false;
  int droneCount = 5;
  int maxDroneCapacity = 40;
  
  num initialWidth;
  num initialHeight;
  num prevWidthScale = 1;
  num prevHeightScale = 1;
  num scaleWidth = 1;
  num scaleHeight = 1;
  
  generateDrones() {
    for (int x = 0; x < droneCount; x++) {
      drones.add(new Drone());
      scaleEntity(drones.last, scaleWidth, scaleHeight);
    }
  }
  
  resize() {
    if (created) {
      scaleWidth = canvas.width / initialWidth;
      scaleHeight = canvas.height / initialHeight;
      unscaleEntity(player, prevWidthScale, prevHeightScale);
      scaleEntity(player, scaleWidth, scaleHeight);
      for (Drone drone in drones) {
        unscaleEntity(drone, prevWidthScale, prevHeightScale);
        scaleEntity(drone, scaleWidth, scaleHeight);
      }
      prevWidthScale = scaleWidth;
      prevHeightScale = scaleHeight;
    }
  }
  
  scaleEntity(Entity entity, num scaleWidth, num scaleHeight) {
    entity.x *= scaleWidth;
    entity.y *= scaleHeight;
    entity.width *= scaleWidth;
    entity.height *= scaleHeight;
  }
  
  unscaleEntity(Entity entity, num scaleWidth, num scaleHeight) {
    entity.x /= scaleWidth;
    entity.y /= scaleHeight;
    entity.width /= scaleWidth;
    entity.height /= scaleHeight;
  }
  
  
  start() {
    initialWidth = canvas.width;
    initialHeight = canvas.height;
    player.healTo(100);
    player.speed = 1.4;
    player.x = (canvas.width / 2) - (player.width / 2); ///Bang in the centre
    player.y = (canvas.height / 2) - (player.height / 2);
    created = true;
    generateDrones();
  }
  

  end() {
    drones.clear();
  }
  
  draw() {
    if (created) {
      for (Drone drone in drones) {
        drone.draw();
      }
      player.draw();
    }
  }
  
  droneSpawnFactory() {
    if (drones.length < maxDroneCapacity) {
      if (random.nextInt(100) == 0) {
        generateDrones();
      }
    }
  }
  
  droneUpgradeFactory(Drone toBeUpgraded, bool killedCleaverDrone, Drone upgrade) {
    upgrade.x = toBeUpgraded.x;
    upgrade.y = toBeUpgraded.y;
    if (killedCleaverDrone) {
      upgrade.width = (toBeUpgraded.width * 1.2).toInt();
      upgrade.height = (toBeUpgraded.height * 1.2).toInt();
    } else {
      upgrade.width = toBeUpgraded.width;
      upgrade.height = toBeUpgraded.height;
    }
    upgrade.speed = toBeUpgraded.speed;
    drones.remove(toBeUpgraded);
    drones.add(upgrade);
  }
  
  droneLogic() {
    if (created) {
      droneSpawnFactory();
      for (Drone drone in drones.toList()) {
        if (drone.outOfBounds()) {
          drone.dead = true;
          /*num widthShare = drone.width / (drones.length - 1);
          num heightShare = drone.width / (drones.length - 1);
          for (Drone dron in drones) {
            if (identical(drone, dron)) continue;
            dron.width += widthShare;
            dron.height += heightShare;
          }*/
        }
        if (drone.dead) {
          if (drone.murderer != null) {
            (drone.murderer as Drone).engorge(drone);
            switch(drone.murderer.runtimeType){
              case PlayerDrone:
                if (drone.murderer.speed > player.speed) {
                  player.speed = drone.murderer.speed;
                  upgradeSpeed(player.speed);
                } else if (drone.murderer.speed < player.speed) {
                  drone.murderer.speed = player.speed;
                }
                break;
              case Drone:
                if (drone.murderer.speed > 1.8 && random.nextInt(10 + drone.murderer.width.toInt()) == 0) {
                  droneUpgradeFactory(drone.murderer, drone is! CleaverDrone, new CleaverDrone());
                }
                break;
            }
          }
          drones.remove(drone);
        }
        if (drone is CleaverDrone) {
          (drone as CleaverDrone).attack(closestTarget(drone));
        } else {
          drone.move();
        }
        if (drone.shieldCharged) {
          for (Drone targetDrone in drones.toList()) {
            if (drone is PlayerDrone && targetDrone is PlayerDrone) continue;
            if (drone.inProximity(targetDrone, drone.width * 1.5)) {
              targetDrone.damage(1, drone);
            }
          }
          if (drone.inProximity(player, drone.width * 1.5)) {
            player.damage(1, drone);
          }
        }
      }
      
      if (player.radius > 0) {
        for (Drone drone in drones) {
          if (drone is PlayerDrone) continue;
          if (player.inProximity(drone, player.radius)) {
            if (drone.speed > player.speed) {
              player.speed = drone.speed;
              upgradeSpeed(player.speed);
            } else if (drone.speed < player.speed) {
              drone.speed = player.speed;
            }
            droneUpgradeFactory(drone, false, new PlayerDrone());
          }
        }
      }
      
    } 
  }
  
  upgradeSpeed(num newSpeed) {
    for (PlayerDrone drone in drones.where((element) => element is PlayerDrone)) {
      drone.speed = newSpeed;
    }
  }
  
  LiveEntity closestTarget(LiveEntity entity) {
    LiveEntity currentClosest = null;
    num totalDistance = 5000;
    for (Drone drone in drones) {
      if (identical(drone, entity)) continue;
      num xDistance = (drone.x - entity.x).abs();
      num yDistance = (drone.y - entity.y).abs();
      if (xDistance + yDistance < totalDistance) {
        totalDistance = xDistance + yDistance;
        currentClosest = drone;
      }
    }
    return currentClosest;
  }
  
  keyInput(keys) {
    if (created) {
      player.input(keys);
      for (PlayerDrone drone in drones.where((element) => element is PlayerDrone)) {
        drone.input(keys);
      }
    }
  }
}
