library empirical;

import 'dart:html';

part 'package:empirical/src/style_interface.dart';
part 'package:empirical/src/menu_interface.dart';

abstract class Empirical {
  var paused = false, started = false;
  var keys = new List<bool>.filled(250, false);

  Empirical() {
    loadStyles(); ///Loads necessary styles etc.
    loadInterface();
    loadEvents();
    window.requestAnimationFrame(_tick);
  }
  
  /**
   * A function that enables the super class to override so the default 
   * Interface Menus can be edited allowing for much more customized games.
   * super.Interface() should almost definitely be called, if overwritten.
   */
  loadInterface() {
    createAllMenus();
    _resize();
    showMenu(getMenu(MAIN));
  }

  /**
   * Controls the logic that occurs when an exit button is pressed on
   * certain types of menus.
   * Including: MainMenu, PauseMenu and OptionsMenu by default.
   */
  exitMenu() {
    switch (currentMenu.runtimeType) {
      case MainMenu:
        window.history.back();
        return;
      case PauseMenu:
        _end();
        showMenu(getMenu(MAIN));
        return;
      case OptionsMenu:
      default:
        showMenu(getMenu(MAIN));
        return;
    }
  }

  loadEvents() {
    ///Alert when a key link is missing, as this may destroy functionality.
    final startButton = query('#startbutton');
    final pauseButton = query('#pausebutton');
    final optionsButton = query('#optionsbutton');
    final exitButtons = queryAll('#exitbutton');
    
    if (startButton != null) {
      startButton.onClick.listen((event) {
        showMenu(getMenu(GAME));
        _start();
      });
    } 
    if (pauseButton != null) {
      pauseButton.onClick.listen((event) {
        _pause();
      });
    }
    if (optionsButton != null) {
      optionsButton.onClick.listen((event) {
        showMenu(OPTIONS);
      });
    }
    if (exitButtons != null) {
      exitButtons.forEach((element) {
        element.onClick.listen((event) {
          exitMenu();
        });
      });
    }
    
    if (query('#mobiletoggle') != null && query('#fullscreentoggle') != null) { 
      query('#mobiletoggle').onClick.listen((event) {
          query('#fullscreentoggle').disabled = !query('#fullscreentoggle').disabled;
          toggleCanvasMobile();
          _resize();
      });
    
      query('#fullscreentoggle').onClick.listen((event) {
        query('#mobiletoggle').disabled = !query('#mobiletoggle').disabled;
        toggleCanvasMaximization();
        _resize();
      });
    }
    window.onKeyUp.listen((event) {
      if (!started) return;
      keys[event.keyCode] = false;
      _keyInput();
    });
    window.onKeyDown.listen((event) {
      if (!started) return;
      keys[event.keyCode] = true;
      _keyInput();
    });
    canvas.onClick.listen((event) {
      if (!paused && started) {
        var coords = relMouseCoords(event);
        clickGame(coords.x, coords.y);
      }
    });
    window.onResize.listen((event) {
      _resize();
    });
  }
  
  void _keyInput() {
    if (keys[80]) {
      _pause();
      return;
    }
    if (keys[82]) {
      if (!paused) restartGame();
      return;
    }
  }
  
  Point relMouseCoords(event){
    var totalOffsetX = canvas.offsetLeft - canvas.scrollLeft;
    var totalOffsetY = canvas.offsetTop - canvas.scrollTop;
    var canvasX = event.pageX - totalOffsetX;
    var canvasY = event.pageY - totalOffsetY;
    return new Point(canvasX, canvasY);
}

  _tick(num time) {
    if (!paused && started) {
      _draw();
      tickGame();
    }
    window.requestAnimationFrame(_tick);
  }
  
  _draw() {
    context.fillStyle = 'black';
    context.fillRect(0, 0, canvas.width, canvas.height);
    drawGame();
  }
  
  _start() {
    paused = false;
    started = true;
    startGame();
  }
  
  _end() {
    started = false;
    endGame();
  }
  
  _pause() {
    paused = !paused;
    if (paused && started) {
      showMenu(getMenu(PAUSE));
    } else {
      showMenu(getMenu(GAME));
    }
    pauseGame();
  }
  
  _resize() {
    resizeCanvas();
    resizeGame();
    _draw(); ///Experiment, requires there to be no logic on draw function.
  }
  
  restartGame() {
    endGame();
    startGame();
  }
  
  drawGame(); 
  startGame();
  endGame();
  pauseGame();
  resizeGame();
  tickGame();
  clickGame(x, y);
}